=== modified file 'drizzled/error.cc'
--- drizzled/error.cc	2011-01-06 16:43:31 +0000
+++ drizzled/error.cc	2011-01-11 05:13:54 +0000
@@ -479,6 +479,8 @@
   ADD_ERROR_MESSAGE(ER_CORRUPT_TABLE_DEFINITION_UNKNOWN_COLLATION, N_("Collation '%s' for table %s is invalid/unknown"));
 
   ADD_ERROR_MESSAGE(ER_TABLE_DROP, N_("Cannot drop table '%s'"));
+  ADD_ERROR_MESSAGE(ER_TABLE_DROP_ERROR_OCCURRED, N_("Error occurred while dropping table '%s'"));
+  ADD_ERROR_MESSAGE(ER_TABLE_PERMISSION_DENIED, N_("Permission denied to create '%s'"));
 
   ADD_ERROR_MESSAGE(ER_SCHEMA_DOES_NOT_EXIST, N_("Schema does not exist: %s"));
   ADD_ERROR_MESSAGE(ER_ALTER_SCHEMA, N_("Error altering schema: %s"));

=== modified file 'drizzled/error_t.h'
--- drizzled/error_t.h	2011-01-06 16:43:31 +0000
+++ drizzled/error_t.h	2011-01-11 05:13:54 +0000
@@ -784,6 +784,8 @@
   ER_ASSERT,
   ER_ASSERT_NULL,
   ER_TABLE_DROP,
+  ER_TABLE_DROP_ERROR_OCCURRED,
+  ER_TABLE_PERMISSION_DENIED,
   ER_INVALID_CAST_TO_BOOLEAN,
 
   // Leave ER_INVALID_BOOLEAN_VALUE as LAST, and force people to use tags

=== modified file 'drizzled/plugin/schema_engine.cc'
--- drizzled/plugin/schema_engine.cc	2011-01-04 22:27:38 +0000
+++ drizzled/plugin/schema_engine.cc	2011-01-11 05:13:54 +0000
@@ -219,7 +219,7 @@
     table::Cache::singleton().removeTable(&session, *it,
                                           RTFC_WAIT_OTHER_THREAD_FLAG |
                                           RTFC_CHECK_KILLED_FLAG);
-    if (plugin::StorageEngine::dropTable(session, *it))
+    if (not plugin::StorageEngine::dropTable(session, *it))
     {
       my_error(ER_TABLE_DROP, *it);
       return false;

=== modified file 'drizzled/plugin/storage_engine.cc'
--- drizzled/plugin/storage_engine.cc	2011-01-06 16:43:31 +0000
+++ drizzled/plugin/storage_engine.cc	2011-01-11 05:13:54 +0000
@@ -435,60 +435,91 @@
   return true;
 }
 
-/**
-   returns ENOENT if the file doesn't exists.
-*/
-int StorageEngine::dropTable(Session& session,
-                             const TableIdentifier &identifier)
-{
-  int error= 0;
-  int error_proto;
-  message::table::shared_ptr src_proto;
-  StorageEngine *engine;
-
-  error_proto= StorageEngine::getTableDefinition(session, identifier, src_proto);
-
-  if (error_proto == ER_CORRUPT_TABLE_DEFINITION)
-  {
-    std::string error_message;
-    identifier.getSQLPath(error_message);
-
-    my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0),
-             error_message.c_str(),
-             src_proto->InitializationErrorString().c_str());
-
-    return ER_CORRUPT_TABLE_DEFINITION;
-  }
-
-  if (src_proto)
-    engine= StorageEngine::findByName(session, src_proto->engine().name());
-  else
-    engine= StorageEngine::findByName(session, "");
-
-  if (not engine)
-  {
-    std::string error_message;
-    identifier.getSQLPath(error_message);
-
-    my_error(ER_CORRUPT_TABLE_DEFINITION, MYF(0), error_message.c_str(), "");
-
-    return ER_CORRUPT_TABLE_DEFINITION;
-  }
-
-  error= StorageEngine::dropTable(session, *engine, identifier);
-
-  if (error_proto && error == 0)
-    return 0;
-
-  return error;
-}
-
-int StorageEngine::dropTable(Session& session,
-                             StorageEngine &engine,
-                             const TableIdentifier &identifier)
-{
-  int error;
-
+class DropTableByIdentifier: public std::unary_function<EngineVector::value_type, bool>
+{
+  Session::reference session;
+  TableIdentifier::const_reference identifier;
+  drizzled::error_t &error;
+
+public:
+
+  DropTableByIdentifier(Session::reference session_arg,
+                        TableIdentifier::const_reference identifier_arg,
+                        drizzled::error_t &error_arg) :
+    session(session_arg),
+    identifier(identifier_arg),
+    error(error_arg)
+  { }
+
+  result_type operator() (argument_type engine)
+  {
+    if (not engine->doDoesTableExist(session, identifier))
+      return false;
+
+    int local_error= engine->doDropTable(session, identifier);
+
+
+    if (not local_error)
+      return true;
+
+    switch (local_error)
+    {
+    case HA_ERR_NO_SUCH_TABLE:
+    case ENOENT:
+      error= static_cast<drizzled::error_t>(HA_ERR_NO_SUCH_TABLE);
+      return false;
+
+    default:
+      error= static_cast<drizzled::error_t>(local_error);
+      return true;
+    }
+  } 
+};
+
+
+bool StorageEngine::dropTable(Session::reference session,
+                              TableIdentifier::const_reference identifier,
+                              drizzled::error_t &error)
+{
+  error= EE_OK;
+
+  EngineVector::const_iterator iter= std::find_if(vector_of_engines.begin(), vector_of_engines.end(),
+                                                  DropTableByIdentifier(session, identifier, error));
+
+  if (error)
+  {
+    return false;
+  }
+  else if (iter == vector_of_engines.end())
+  {
+    error= ER_BAD_TABLE_ERROR;
+    return false;
+  }
+
+  drizzled::message::Cache::singleton().erase(identifier);
+
+  return true;
+}
+
+bool StorageEngine::dropTable(Session& session,
+                              const TableIdentifier &identifier)
+{
+  drizzled::error_t error;
+
+  if (not dropTable(session, identifier, error))
+  {
+    return false;
+  }
+
+  return true;
+}
+
+bool StorageEngine::dropTable(Session::reference session,
+                              StorageEngine &engine,
+                              TableIdentifier::const_reference identifier,
+                              drizzled::error_t &error)
+{
+  error= EE_OK;
   engine.setTransactionReadWrite(session);
   
   if (unlikely(plugin::EventObserver::beforeDropTable(session, identifier)))
@@ -497,7 +528,8 @@
   }
   else
   {
-    error= engine.doDropTable(session, identifier);
+    error= static_cast<drizzled::error_t>(engine.doDropTable(session, identifier));
+
     if (unlikely(plugin::EventObserver::afterDropTable(session, identifier, error)))
     {
       error= ER_EVENT_OBSERVER_PLUGIN;
@@ -506,7 +538,12 @@
 
   drizzled::message::Cache::singleton().erase(identifier);
 
-  return error;
+  if (error)
+  {
+    return false;
+  }
+
+  return true;
 }
 
 
@@ -554,7 +591,11 @@
                                                  table_message);
     }
 
-    if (error)
+    if (error == ER_TABLE_PERMISSION_DENIED)
+    {
+      my_error(ER_TABLE_PERMISSION_DENIED, identifier);
+    }
+    else if (error)
     {
       std::string path;
       identifier.getSQLPath(path);

=== modified file 'drizzled/plugin/storage_engine.h'
--- drizzled/plugin/storage_engine.h	2011-01-06 16:43:31 +0000
+++ drizzled/plugin/storage_engine.h	2011-01-11 05:13:54 +0000
@@ -32,6 +32,8 @@
 #include "drizzled/cached_directory.h"
 #include "drizzled/plugin/monitored_in_transaction.h"
 
+#include <drizzled/error_t.h>
+
 #include <bitset>
 #include <string>
 #include <vector>
@@ -313,6 +315,7 @@
   friend class StorageEngineDoesTableExist;
   friend class StorageEngineGetSchemaDefinition;
   friend class StorageEngineGetTableDefinition;
+  friend class DropTableByIdentifier;
 
   int renameTable(Session &session, const drizzled::TableIdentifier &from, const drizzled::TableIdentifier &to);
 
@@ -334,11 +337,18 @@
   static void closeConnection(Session* session);
   static void dropDatabase(char* path);
   static bool flushLogs(plugin::StorageEngine *db_type);
-  static int dropTable(Session& session,
-                       const drizzled::TableIdentifier &identifier);
-  static int dropTable(Session& session,
-                       StorageEngine &engine,
-                       const drizzled::TableIdentifier &identifier);
+
+  static bool dropTable(Session& session,
+                        const drizzled::TableIdentifier &identifier);
+  static bool dropTable(Session& session,
+                        const drizzled::TableIdentifier &identifier,
+                        drizzled::error_t &error);
+
+  static bool dropTable(Session& session,
+                        StorageEngine &engine,
+                        TableIdentifier::const_reference identifier,
+                        drizzled::error_t &error);
+
   static void getIdentifiers(Session &session,
                              const SchemaIdentifier &schema_identifier,
                              TableIdentifier::vector &set_of_identifiers);

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2011-01-08 04:23:10 +0000
+++ drizzled/session.cc	2011-01-11 05:13:54 +0000
@@ -1975,7 +1975,7 @@
 
 bool Open_tables_state::rm_temporary_table(const TableIdentifier &identifier, bool best_effort)
 {
-  if (plugin::StorageEngine::dropTable(*static_cast<Session *>(this), identifier))
+  if (not plugin::StorageEngine::dropTable(*static_cast<Session *>(this), identifier))
   {
     if (not best_effort)
     {
@@ -1993,14 +1993,15 @@
 
 bool Open_tables_state::rm_temporary_table(plugin::StorageEngine *base, const TableIdentifier &identifier)
 {
+  drizzled::error_t error;
   assert(base);
 
-  if (plugin::StorageEngine::dropTable(*static_cast<Session *>(this), *base, identifier))
+  if (not plugin::StorageEngine::dropTable(*static_cast<Session *>(this), *base, identifier, error))
   {
     std::string path;
     identifier.getSQLPath(path);
     errmsg_printf(ERRMSG_LVL_WARN, _("Could not remove temporary table: '%s', error: %d"),
-                  path.c_str(), errno);
+                  path.c_str(), error);
 
     return true;
   }

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2011-01-08 11:27:50 +0000
+++ drizzled/sql_base.cc	2011-01-11 05:13:54 +0000
@@ -730,7 +730,7 @@
       that something has happened.
     */
     unlink_open_table(table);
-    plugin::StorageEngine::dropTable(*this, identifier);
+    (void)plugin::StorageEngine::dropTable(*this, identifier);
   }
 }
 

=== modified file 'drizzled/sql_insert.cc'
--- drizzled/sql_insert.cc	2011-01-08 11:27:50 +0000
+++ drizzled/sql_insert.cc	2011-01-11 05:13:54 +0000
@@ -1606,7 +1606,7 @@
 
           if (concurrent_table->reopen_name_locked_table(create_table, session))
           {
-            plugin::StorageEngine::dropTable(*session, identifier);
+            (void)plugin::StorageEngine::dropTable(*session, identifier);
           }
           else
           {
@@ -1615,7 +1615,7 @@
         }
         else
         {
-          plugin::StorageEngine::dropTable(*session, identifier);
+          (void)plugin::StorageEngine::dropTable(*session, identifier);
         }
       }
       else

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2011-01-07 21:56:29 +0000
+++ drizzled/sql_table.cc	2011-01-11 05:13:54 +0000
@@ -212,25 +212,28 @@
       }
       else
       {
-        error= plugin::StorageEngine::dropTable(*session, identifier);
+        drizzled::error_t local_error;
 
         /* Generate transaction event ONLY when we successfully drop */ 
-        if (error == 0)
+        if (plugin::StorageEngine::dropTable(*session, identifier, local_error))
         {
           TransactionServices &transaction_services= TransactionServices::singleton();
           transaction_services.dropTable(session, identifier, if_exists);
         }
-
-        if ((error == ENOENT || error == HA_ERR_NO_SUCH_TABLE) && if_exists)
-        {
-          error= 0;
-          session->clear_error();
-        }
-
-        if (error == HA_ERR_ROW_IS_REFERENCED)
-        {
-          /* the table is referenced by a foreign key constraint */
-          foreign_key_error= true;
+        else
+        {
+          if (local_error == HA_ERR_NO_SUCH_TABLE and if_exists)
+          {
+            error= 0;
+            session->clear_error();
+          }
+
+          if (local_error == HA_ERR_ROW_IS_REFERENCED)
+          {
+            /* the table is referenced by a foreign key constraint */
+            foreign_key_error= true;
+          }
+          error= local_error;
         }
       }
 

=== modified file 'drizzled/table/instance.cc'
--- drizzled/table/instance.cc	2010-12-30 18:10:25 +0000
+++ drizzled/table/instance.cc	2011-01-11 05:13:54 +0000
@@ -326,9 +326,11 @@
     }
 
     TableIdentifier identifier(getShare()->getSchemaName(), getShare()->getTableName(), getShare()->getTableName());
+    drizzled::error_t ignored;
     plugin::StorageEngine::dropTable(*in_use,
                                      *getShare()->getEngine(),
-                                     identifier);
+                                     identifier,
+                                     ignored);
 
     delete cursor;
   }

=== modified file 'plugin/function_engine/function.h'
--- plugin/function_engine/function.h	2010-12-18 04:43:40 +0000
+++ plugin/function_engine/function.h	2011-01-11 05:13:54 +0000
@@ -51,12 +51,12 @@
                     const drizzled::TableIdentifier &,
                     drizzled::message::Table&)
   {
-    return EPERM;
+    return drizzled::ER_TABLE_PERMISSION_DENIED;
   }
 
   int doDropTable(drizzled::Session&, const drizzled::TableIdentifier&)
   { 
-    return EPERM; 
+    return HA_ERR_NO_SUCH_TABLE; 
   }
 
   virtual drizzled::Cursor *create(drizzled::Table &table);

=== modified file 'plugin/schema_engine/schema.h'
--- plugin/schema_engine/schema.h	2010-12-11 17:34:45 +0000
+++ plugin/schema_engine/schema.h	2011-01-11 05:13:54 +0000
@@ -81,7 +81,7 @@
 
   int doRenameTable(drizzled::Session&, const drizzled::TableIdentifier&, const drizzled::TableIdentifier&)
   {
-    return EPERM;
+    return HA_ERR_NO_SUCH_TABLE;
   }
 
   int doCreateTable(drizzled::Session&,
@@ -89,12 +89,12 @@
                     const drizzled::TableIdentifier&,
                     drizzled::message::Table&)
   {
-    return EPERM;
+    return drizzled::ER_TABLE_PERMISSION_DENIED;
   }
 
   int doDropTable(drizzled::Session&, const drizzled::TableIdentifier&)
   {
-    return 0;
+    return HA_ERR_NO_SUCH_TABLE;
   }
 
   const char **bas_ext() const 
@@ -107,6 +107,7 @@
                           uint64_t *,
                           uint64_t *)
   {}
+
   void doGetTableIdentifiers(drizzled::CachedDirectory &directory,
                              const drizzled::SchemaIdentifier &schema_identifier,
                              drizzled::TableIdentifier::vector &set_of_identifiers);

=== modified file 'plugin/tableprototester/tableprototester.cc'
--- plugin/tableprototester/tableprototester.cc	2010-12-02 01:39:53 +0000
+++ plugin/tableprototester/tableprototester.cc	2011-01-11 05:13:54 +0000
@@ -95,7 +95,7 @@
 
   int doRenameTable(Session&, const drizzled::TableIdentifier&, const drizzled::TableIdentifier&)
   {
-    return EPERM;
+    return HA_ERR_NO_SUCH_TABLE;
   }
 
   void doGetTableIdentifiers(drizzled::CachedDirectory &directory,
@@ -153,7 +153,7 @@
 
 int TableProtoTesterEngine::doDropTable(Session&, const drizzled::TableIdentifier&)
 {
-  return EPERM;
+  return HA_ERR_NO_SUCH_TABLE;
 }
 
 static void fill_table1(message::Table &table)

=== modified file 'plugin/tableprototester/tests/r/drop_table.result'
--- plugin/tableprototester/tests/r/drop_table.result	2010-12-22 21:50:32 +0000
+++ plugin/tableprototester/tests/r/drop_table.result	2011-01-11 05:13:54 +0000
@@ -1,2 +1,2 @@
 DROP TABLE t1;
-ERROR HY000: Corrupt or invalid table definition for test.t1: 
+Got one of the listed errors

=== modified file 'plugin/tableprototester/tests/t/drop_table.test'
--- plugin/tableprototester/tests/t/drop_table.test	2010-08-26 21:56:15 +0000
+++ plugin/tableprototester/tests/t/drop_table.test	2011-01-11 05:13:54 +0000
@@ -1,2 +1,2 @@
---error ER_CORRUPT_TABLE_DEFINITION
+--error ER_CORRUPT_TABLE_DEFINITION,ER_BAD_TABLE_ERROR
 DROP TABLE t1;

=== modified file 'tests/r/broken_table_proto_file.result'
--- tests/r/broken_table_proto_file.result	2010-12-22 21:50:32 +0000
+++ tests/r/broken_table_proto_file.result	2011-01-11 05:13:54 +0000
@@ -4,4 +4,4 @@
 show create table t1;
 ERROR HY000: Corrupt or invalid table definition for test/t1.dfe: name, schema, type, engine, creation_timestamp, update_timestamp
 drop table if exists t1;
-ERROR HY000: Corrupt or invalid table definition for test/t1.dfe: name, schema, type, engine, creation_timestamp, update_timestamp
+Got one of the listed errors

=== modified file 'tests/r/create.result'
--- tests/r/create.result	2011-01-06 16:43:31 +0000
+++ tests/r/create.result	2011-01-11 05:13:54 +0000
@@ -1460,7 +1460,7 @@
 ) ENGINE=DEFAULT COLLATE = utf8_general_ci
 drop table имя_таблицы_в_кодировке_утф8_длиной_больше_чем_48;
 create table t1 like data_dictionary.processlist;
-ERROR HY000: Can't create table 'test.t1' (errno: 1)
+Got one of the listed errors
 create table t1 like data_dictionary.processlist engine=innodb;
 show create table t1;
 Table	Create Table
@@ -1477,7 +1477,7 @@
 ) ENGINE=InnoDB COLLATE = utf8_general_ci
 drop table t1;
 create temporary table t1 like data_dictionary.processlist;
-ERROR HY000: Can't create table 'test.#t1' (errno: 138)
+Got one of the listed errors
 create temporary table t1 like data_dictionary.processlist engine=myisam;
 show create table t1;
 Table	Create Table

=== modified file 'tests/t/broken_table_proto_file.test'
--- tests/t/broken_table_proto_file.test	2010-10-07 21:20:57 +0000
+++ tests/t/broken_table_proto_file.test	2011-01-11 05:13:54 +0000
@@ -13,7 +13,7 @@
 SHOW TABLE STATUS like 't1';
 --error ER_CORRUPT_TABLE_DEFINITION
 show create table t1;
---error ER_CORRUPT_TABLE_DEFINITION
+--error ER_CORRUPT_TABLE_DEFINITION,ER_BAD_TABLE_ERROR
 drop table if exists t1;
 system rm -f $DRIZZLETEST_VARDIR/master-data/local/test/t1.dfe;
 

=== modified file 'tests/t/create.test'
--- tests/t/create.test	2010-12-23 22:20:46 +0000
+++ tests/t/create.test	2011-01-11 05:13:54 +0000
@@ -1213,12 +1213,12 @@
 #
 # Bug#25629 CREATE TABLE LIKE does not work with INFORMATION_SCHEMA
 #
---error ER_CANT_CREATE_TABLE
+--error ER_CANT_CREATE_TABLE,ER_TABLE_PERMISSION_DENIED
 create table t1 like data_dictionary.processlist;
 create table t1 like data_dictionary.processlist engine=innodb;
 show create table t1;
 drop table t1;
---error ER_CANT_CREATE_TABLE
+--error ER_CANT_CREATE_TABLE,ER_TABLE_PERMISSION_DENIED
 create temporary table t1 like data_dictionary.processlist;
 create temporary table t1 like data_dictionary.processlist engine=myisam;
 show create table t1;

